# This file is part of Pantry, command-line nutrient analysis.
# 
# Version 20, released Friday, August 24, 2007.
# 
# Copyright 2007 Omari Norman.
# 
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the
# Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute,
# sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
# 
# The above copyright notice and this permission notice shall
# be included in all copies or substantial portions of the
# Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 
# Pantry's "built-in" XML validator is a modified version of
# xmlproc.  xmlproc is Copyright 1998-2000 by Lars Marius
# Garshol, Oslo, Norway.
# 
# All Rights Reserved
# 
# Permission to use, copy, modify, and distribute this
# software and its documentation for any purpose and without
# fee is hereby granted, provided that the above copyright
# notice appear in all copies and that modified copies are
# clearly marked as such.
# 
# LARS MARIUS GARSHOL DISCLAIMS ALL WARRANTIES WITH REGARD TO
# THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL LARS MARIUS
# GARSHOL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
# DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

from __future__ import division

class Reports(object):
    """ Print out food reports. """
    
    SUMMARY_REPORTS = ['sum', 'groups', 'list']
    NUT_REPORTS = ['sum', 'nuts', 'debug']

    def __init__(self, reports, prog, outFile, errFile, sortFields = '', 
            sortOrders = []):
        from pantry.lib import foods
        from pantry.lib import io
        import sys

        reports = reports.split('-')
        self.foodReports = []
        self.summaryReports = []
        self.sortOrders = sortOrders
        self.prog = prog
        availReports = vars(Reports).keys()

        if not outFile or outFile == '-':
            self.out = io.puni(sys.stdout)
        else:
            fh = open(outFile, 'w')
            self.out = io.puni(fh)

        if not errFile or errFile == '-':
            self.stderr = io.puni(sys.stderr)
        else:
            fh = open(errFile, 'w')
            self.stderr = io.puni(fh)
        self.err = io.ErrorPrinter(self.stderr, prog, io.WARNING)

        for report in reports:
            possRpts = [ rptName for rptName in availReports
                    if rptName.startswith(report) ]
            if len(possRpts) != 1:
                self.err('report name %s does not exist or is ambiguous.' %
                        report)
            else:
                if possRpts[0] in self.SUMMARY_REPORTS:
                    self.summaryReports.append(possRpts[0])
                else:
                    self.foodReports.append(possRpts[0])

        self.sortList = []
        for char in sortFields:
            for traitName in foods.TRAITSBYNAME:
                found = False
                if traitName.startswith(char):
                    self.sortList.append(traitName)
                    found = True
                    break
                elif traitName.startswith(char.lower()):
                    self.sortList.append('-' + traitName)
                    found = True
                    break
            if not found:
                self.err = ("sort trait %s not recognized" % char)

        self.needNuts = False
        for report in self.foodReports + self.summaryReports:
            if report in self.NUT_REPORTS:
                self.needNuts = True
                return

    def printReports(self, unsortedFoods, nutList):
        from pantry.lib import foods
        from pantry.lib import cabinets
        self.nutList = nutList
        self.goalReporter = cabinets.GoalReporter(nutList, unsortedFoods)
        self.foods = unsortedFoods

        if self.foodReports:
            if self.sortList:
                foodsToPrint = sorted(unsortedFoods.values(), 
                        foods.CmpFoods(self.sortList, self.sortOrders))
            else:
                foodsToPrint = unsortedFoods.values()

            for food in foodsToPrint:
                for report in self.foodReports:
                    getattr(self, report)(food)

        for report in self.summaryReports:
            getattr(self, report)()

    def names(self, food):
        self.out(food.traits['name'] + '\n')

    def makes(self, food):
        try:
            if food.rTraits['yieldDesc']:
                self.out(food.rTraits['yieldDesc'] + ' ')
        except AttributeError:
            self.err('food %s is not a recipe.' % food.traits['name'])
            return
        self.out('(%.0fg)\n' % food.totalGrams)

    def ingredients(self, food):
        if not hasattr(food, 'ingredients'):
            self.err('food %s is not a recipe.' % food.traits['name'])
            return
        for ingr in food.ingredients:
            grams = ingr.grams
            oz = ingr.grams / 28.35
            self.out('%s %s (%.3foz, %.0fg) %s' % (ingr.traits['qty'],
                    ingr.traits['unit'], oz, grams, ingr.traits['name']))
            if ingr.traits['comment']:
                self.out(' (%s)' % ingr.traits['comment'])
            self.out('\n')

    def directions(self, food):
        import re
        if not hasattr(food, 'ingredients'):
            self.err('food %s is not a recipe.' % food.traits['name'])
            return
        directions = food.rTraits['directions'].rstrip()
        directions = re.sub(r'<p[^>]*>', '', directions)
        directions = directions.replace('\n', ' ')
        directions = directions.replace('\r\n', ' ')
        directions = directions.replace('\r', ' ')
        directions = re.sub(r'\\t*', ' ', directions)
        directions = re.sub(' {2,}', ' ', directions)
        directions = re.sub(r'</p>', '\n', directions).lstrip()
        self.out(directions)

    def recipe(self, food):
        """ Prints recipe in plain text. 
        
        Prints warning for non-recipe foods. """
        if not hasattr(food, 'rTraits'):
            self.err('food %s is not a recipe.' % food.traits['name'])
            return
        
        self.names(food)
        self.makes(food)
        self.blank(food)
        self.ingredients(food)
        self.blank(food)
        self.directions(food)
        
    def paste(self, food):
        OPTEXACT='-x'
        OPTNAME='-n'
        OPTCHANGEUNIT='-U'

        name = food.traits['name']
        name = "'%s'" % name.replace("'", "'\\''")
        availUnits = food.units
        availUnits = [ "'%s'" % unit.replace("'", "'\\''")
                for unit in availUnits ]

        for availUnit in availUnits:
            self.out(' '.join([OPTEXACT, OPTNAME, name, 
                OPTCHANGEUNIT, availUnit, '\n']))

    def traits(self, food):
        from pantry.lib import uniconv

        traits = food.traits

        self.out(traits['name'] + '\n')
        if traits['group']:
            self.out('Group: %s\n' % traits['group'])

        atStartOfLine = True
        for traitName in ['date', 'meal', 'order']:
            if traits[traitName]:
                self.out('%s: %s ' % 
                        (traitName.capitalize(), traits[traitName]))
                atStartOfLine = False
        if not atStartOfLine:
            self.out('\n',)

        if traits['comment']:
            self.out('Comment: %s\n' % traits['comment'])

        if uniconv.convertUni(traits['pctRefuse']):
            self.out(('Refuse: %s percent %s\n' %
                    (traits['pctRefuse'], traits['refDesc'])))

        self.out(('%s %s (%.0fg)\n' 
                % (traits['qty'], traits['unit'], food.grams)))

    def measures(self, food):
        units = food.units
        for unit, grams in units.items():
            self.out('   %s (%.0fg)\n' % (unit, grams))

    def units(self, food):
        from pantry.lib import foods
        units = set(food.units).difference(foods.ABSOLUTES)
        for unit in units:
            self.out('   %s\n' % unit)

    def debug(self, food):
        self.out(str(food) + '\n')

    def list(self):
        """ Prints nutList in nice format. """
        from pantry.lib import foods

        format = "%-25s %-14s %-6s\n"
        headerLabels = ('Nutrient Name', 'Units', 'Goal')
        headerString = format % headerLabels
        self.out(headerString)
        self.out("-" * len(headerString) + '\n')

        for nutName, goal in self.nutList:
            nutUnits = foods.NUTSBYNAME[nutName].units
            self.out(format % (nutName, nutUnits, goal))

    def _nutReport(self, food):
        """ Prints nutrient report for food.

        If food is None, prints sum nutrient report.
        """

        headerFormat = "%-25s %-14s %-6s"
        headerLabels = ['Nutrient', 'Amount', '%G' ]
        if food:
            headerFormat += ' %-6s'
            headerLabels.append('%TOT')
        else:
            headerFormat = 'SUM:\n%s' % headerFormat
        headerFormat += '\n'
        headerString = headerFormat % tuple(headerLabels)
        self.out(headerString)
        self.out("-" * len(headerString) + '\n')
        lineFormat = "%-25s %-4s %-10s %-6s"
        if food:
            lineFormat += ' %-6s\n'
            data = self.goalReporter[food.hash]
        else:
            lineFormat += '\n'
            data = self.goalReporter.totals

        for line in data:
            toPrint = [line['nutID'][0], line['nutAmt'], 
                    line['nutID'][1], line['pctGoal']]
            if food:
                toPrint.append(line['pctTotal'])

            self.out(lineFormat % tuple(toPrint))

    def nuts(self, food):
        self._nutReport(food)

    def sum(self):
        self._nutReport(None)

    def blank(self, food):
        self.out('\n')

    def groups(self):
        """ Prints group names and number of foods in each group. """
        groupDict = self.foods.groups
        self.out('  %-35s %7s\n' % ('Group', 'No. of Foods'))
        self.out('-' * 60 + '\n')
        for group in sorted(groupDict):
            self.out('  %-35s %7u\n' % (group, len(groupDict[group])))
        self.out('-' * 60 + '\n')
        self.out('  %-35s %7u foods total\n' % ('', len(self.foods)))
        self.out('  %-35s %7u groups total\n' % ('', len(groupDict)))

    def getNeedNuts(self):
        return self.needNuts
